@@ -1,10 +1,11 @@
 
-__version__ = "10.3.1"
+__version__ = "10.3.5"
 __author__ = "Alex Al-Saffar"
 __date__ = "2023-01-13"
 docs = r"https://crocodile.readthedocs.io/en/latest/"
 git = r"https://github.com/thisismygitrepo/crocodile"
 pypi = r"https://pypi.org/project/crocodile"
 release_notes = """
-Added rclone cloud functionality
+added ascci art for windows
 """
+
