@@ -65,26 +65,17 @@ api = GDriveAPI()
     @staticmethod
     def sftp(self: RemoteMachine):
         assert self.ssh.sftp is not None, f"SFTP is not available for this machine `{self}`. Consider using different `transfer_method` other than `sftp`."
-        self.ssh.run_py(f"tb.P(r'{ResourceManager.shell_script_path_log}').expanduser().create(parents_only=True).delete(sure=True).write_text(r'{self.path_dict.shell_script_path.collapseuser().as_posix()}')")
+        self.ssh.run_py(f"tb.P(r'{ResourceManager.shell_script_path_log}').expanduser().create(parents_only=True).delete(sure=True).write_text(r'{self.path_dict.shell_script_path.collapseuser().as_posix()}')", desc="Logging latest shell script path on remote.", verbose=False)
         if self.copy_repo: self.ssh.copy_from_here(self.repo_path, z=True, overwrite=True)
         tb.L(self.data).apply(lambda x: self.ssh.copy_from_here(x, z=True if tb.P(x).is_dir() else False, r=False, overwrite=True))
 
         self.ssh.copy_from_here(self.path_dict.root_dir, z=True)
-        self.ssh.print_summary()
+        # self.ssh.print_summary()
 
         # f"source " if self.ssh.get_remote_machine() != "Windows" else "")
         self.execution_command = ". " + f"{self.path_dict.shell_script_path.collapseuser().as_posix()}"
         # self.ssh.run(f". {self.shell_script_path.collapseuser().as_posix()}", desc="Executing the function")
 
-        if self.open_console:
-            self.ssh.open_console()
-            # send email at start execution time
-            # run_script = f""" pwsh -Command "ssh -t alex@flask-server 'tmux'" """
-            # https://stackoverflow.com/questions/31902929/how-to-write-a-shell-script-that-starts-tmux-session-and-then-runs-a-ruby-scrip
-            # https://unix.stackexchange.com/questions/409861/is-it-possible-to-send-input-to-a-tmux-session-without-connecting-to-it
-        else:
-            pass
-
 
 if __name__ == '__main__':
     pass
