@@ -36,7 +36,7 @@ class HyperParam(tb.Struct):
     def __init__(self, **kwargs):
         super().__init__(
             # ==================== Enviroment =========================
-            name='default_model_name_' + tb.randstr(),
+            name="model_" + tb.randstr(noun=True),
             root=tb.P.tmp(folder="tmp_models"),
             pkg_name='tensorflow',
             device_name=Device.gpu0,
@@ -68,7 +68,7 @@ class HyperParam(tb.Struct):
     @property
     def pkg(self): return __import__("tensorflow") if self.pkg_name == "tensorflow" else (__import__("torch") if self.pkg_name == "torch" else ValueError(f"pkg_name must be either `tensorflow` or `torch`"))
     @property
-    def save_dir(self): return (tb.P(self.root) / self.name).create()
+    def save_dir(self) -> tb.P: return (tb.P(self.root) / self.name).create()
 
     @property
     def device(self):
@@ -366,7 +366,7 @@ class BaseModel(ABC):
                                      validation_data=(x_test, y_test) if val_sample_weights is None else (x_test, y_test, val_sample_weights),
                                      batch_size=self.hp.batch_size, epochs=self.hp.epochs, verbose=1, shuffle=self.hp.shuffle, callbacks=[])
         default_settings.update(kwargs)
-        hist = self.model.fit(**default_settings.dict)
+        hist = self.model.fit(**default_settings.__dict__)
         self.history.append(tb.Struct(copy.deepcopy(hist.history)))  # it is paramount to copy, cause source can change.
         if viz:
             artist = self.plot_loss()
